/* SGContentViewer.m
*  
* Copyright (C) 2004 Free Software Foundation, Inc.
*
* Author: Charles Philip Chan <cpchan@sympatico.ca>
* Date: May 24, 2006
*
* This file is part of the GNUstep Inspector application
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
*/


#import <ShengGuang/SGSound.h>
#include <AppKit/AppKit.h>
#include "SGContentViewer.h"

@implementation SGContentViewer

- (void)dealloc
{
	TEST_RELEASE (soundPath); 
	TEST_RELEASE (sound);
	RELEASE (infoBox);	
	RELEASE (playBox);  
	RELEASE (errLabel);
	RELEASE (indicator);

	RELEASE(infoView);
	RELEASE(length);
	RELEASE(channels);
	RELEASE(nominalOfBitrate);
	RELEASE(upperOfBitrate);
	RELEASE(lowerOfBitrate);
	RELEASE(rate);
	RELEASE(anOgg);
	RELEASE(anMp3);

	[super dealloc];
}

- (id)initWithFrame:(NSRect)frameRect
          inspector:(id)insp
{
	self = [super initWithFrame: frameRect];

	if(self) {
		NSBundle *bundle;
		NSString *imagePath;
		NSImage *image;
		
		infoBox = [[NSBox alloc] initWithFrame: NSMakeRect(3, 130, 256, 146)];	
		[infoBox setBorderType: NSNoBorder];
		[infoBox setTitle: NSLocalizedString(@"Info", @"")];					
		[infoBox setTitlePosition: NSAtTop];
		[infoBox setContentViewMargins: NSMakeSize(0, 0)]; 
		[self addSubview: infoBox];

		NSRect r = [self frame];
        
		r.origin.y = 130;
		r.size.height = 128;
		scrollInfoView = [[NSScrollView alloc] initWithFrame: r];
		[scrollInfoView setBorderType: NSBezelBorder];
		[scrollInfoView setHasHorizontalScroller: YES];
		[scrollInfoView setHasVerticalScroller: YES]; 
		[scrollInfoView setAutoresizingMask: NSViewHeightSizable | NSViewWidthSizable];
		[[scrollInfoView contentView] setAutoresizingMask: NSViewHeightSizable | NSViewWidthSizable];
		[[scrollInfoView contentView] setAutoresizesSubviews: YES];
		[self addSubview: scrollInfoView];

		r = [[scrollInfoView contentView] frame];

		   infoView = [[NSTableView alloc] initWithFrame: r];


		[scrollInfoView setDocumentView: infoView];
		RELEASE (infoView);
		
		playBox = [[NSBox alloc] initWithFrame: NSMakeRect(33, 45, 197, 80)];	
		[playBox setBorderType: NSGrooveBorder];
		[playBox setTitle: NSLocalizedString(@"Player", @"")];
		[playBox setTitlePosition: NSAtTop];
		[playBox setContentViewMargins: NSMakeSize(0, 0)]; 
		[self addSubview: playBox]; 

		bundle = [NSBundle bundleForClass: [self class]];
		
		stopButt = [[NSButton alloc] initWithFrame: NSMakeRect(56, 30, 24, 24)];
		[stopButt setButtonType: NSMomentaryLight];
		[stopButt setImagePosition: NSImageOnly];
		imagePath = [bundle pathForResource: @"stop" ofType: @"tiff" inDirectory: nil];		
		image = [[NSImage alloc] initWithContentsOfFile: imagePath];
		[stopButt setImage: image];
		RELEASE (image);
		[stopButt setTarget:self];
		[stopButt setAction:@selector(buttonsAction:)];
		[playBox addSubview: stopButt]; 
		RELEASE (pauseButt);
    
		pauseButt = [[NSButton alloc] initWithFrame: NSMakeRect(86, 30, 24, 24)];
		[pauseButt setButtonType: NSMomentaryLight];
		[pauseButt setImagePosition: NSImageOnly];
		imagePath = [bundle pathForResource: @"pause" ofType: @"tiff" inDirectory: nil];		
		image = [[NSImage alloc] initWithContentsOfFile: imagePath];
		[pauseButt setImage: image];
		RELEASE (image);
		[pauseButt setTarget:self];
		[pauseButt setAction:@selector(buttonsAction:)];
		[playBox addSubview: pauseButt]; 
		RELEASE (pauseButt);

		playButt = [[NSButton alloc] initWithFrame: NSMakeRect(116, 30, 24, 24)];
		[playButt setButtonType: NSMomentaryLight];
		[playButt setImagePosition: NSImageOnly];
		imagePath = [bundle pathForResource: @"play" ofType: @"tiff" inDirectory: nil];		
		image = [[NSImage alloc] initWithContentsOfFile: imagePath];
		[playButt setImage: image];
		RELEASE (image);
		[playButt setTarget:self];
		[playButt setAction:@selector(buttonsAction:)];
		[playBox addSubview: playButt]; 
		RELEASE (playButt);
		
		indicator = [[NSProgressIndicator alloc] 
						initWithFrame: NSMakeRect(10, 6, 172, 16)];
		[indicator setIndeterminate: YES];
		[playBox addSubview: indicator]; 
								
		editButt = [[NSButton alloc] initWithFrame: NSMakeRect(141, 10, 115, 25)];
		[editButt setButtonType: NSMomentaryLight];
		[editButt setImage: [NSImage imageNamed: @"common_ret.tiff"]];
		[editButt setImagePosition: NSImageRight];
		[editButt setTitle: NSLocalizedString(@"Edit", @"")];
		[editButt setTarget: self];
		[editButt setAction: @selector(editFile:)];	
		[editButt setEnabled: NO];		
		[self addSubview: editButt]; 
		RELEASE (editButt);
    
		errLabel = [[NSTextField alloc] init];	
		[errLabel setFrame: NSMakeRect(5, 162, [self frame].size.width - 10, 25)];
		[errLabel setAlignment: NSCenterTextAlignment];
		[errLabel setFont: [NSFont systemFontOfSize: 18]];
		[errLabel setBackgroundColor: [NSColor windowBackgroundColor]];
		[errLabel setTextColor: [NSColor darkGrayColor]];	
		[errLabel setBezeled: NO];
		[errLabel setEditable: NO];
		[errLabel setSelectable: NO];
		[errLabel setStringValue: NSLocalizedString(@"Invalid Contents", @"")];
    
		soundPath = nil;
		sound = nil;
                
		inspector = insp;
		ws = [NSWorkspace sharedWorkspace];
    
		valid = YES;
	}
	
	return self;
}

- (void)buttonsAction:(id)sender
{
	if (sender == playButt) {
		if (sound) {
			if ([sound isPlaying] && [sound isPaused] == 0) {
				return;
			} else if ([sound isPlaying] && [sound isPaused]) {
				[indicator startAnimation: self];
				[sound resume];
				return;			
			} else			
				[indicator startAnimation: self];
			[sound play];			  
        }
		   	  		
	} else if (sender == pauseButt) {
		if (sound && [sound isPlaying] && [sound isPaused] == 0) {
			[indicator stopAnimation: self];
			[sound pause];
		} 
				
	} else if (sender == stopButt) {
		if (sound && [sound isPlaying]) {
			[indicator stopAnimation: self];
			[sound stop];
			[editButt setEnabled: YES];	
			[[self window] makeFirstResponder: editButt];	  
		}
	}
}

- (void)editFile:(id)sender
{
	NSString *appName = nil, *type = nil;

	[ws getInfoForFile: soundPath application: &appName type: &type];

	if (appName != nil) {
		NS_DURING
		{
			[ws openFile: soundPath withApplication: appName];
		}
		NS_HANDLER
		{
			NSRunAlertPanel(NSLocalizedString(@"error", @""), 
							[NSString stringWithFormat: @"%@ %@!", 
									  NSLocalizedString(@"Can't open ", @""), [soundPath lastPathComponent]],
							NSLocalizedString(@"OK", @""), 
							nil, 
							nil);                                     
		}
		NS_ENDHANDLER  
			}
}

- (void)displayPath:(NSString *)path
{

	SGSound *snd;
	
	if (sound) {
		if ([sound isPlaying]) {
			[sound stop];
			[indicator stopAnimation: self];
		}
		DESTROY (sound);
	}
  
	ASSIGN (soundPath, path);

	if ([self superview]) {      
		[inspector contentsReadyAt: soundPath];
	}

	snd = [[SGSound alloc] initWithContentsOfFile: soundPath
						   byReference: NO];  

	if (snd) {
		ASSIGN (sound, snd);
			  		                
		if (valid == NO) {
			[errLabel removeFromSuperview]; 
			[self addSubview: playBox];   
			valid = YES;
		}		  
		    
		[editButt setEnabled: YES];	
		[[self window] makeFirstResponder: editButt];
		    	
	} else {
		if (valid == YES) {
			DESTROY (sound);
			[playBox removeFromSuperview]; 
			[self addSubview: errLabel]; 
			[editButt setEnabled: NO];		      
			valid = NO;
		}
	}
  
	DESTROY (snd);
  
}

- (void)displayLastPath:(BOOL)forced
{
	if (soundPath) {
		if (forced) {
			[self displayPath: soundPath];
		} else {
			[inspector contentsReadyAt: soundPath];
		}
	}
}

- (void)displayData:(NSData *)data 
             ofType:(NSString *)type
{
}

- (NSString *)currentPath
{
	return soundPath;
}

- (void)stopTasks
{
	if (sound) {
		if ([sound isPlaying]) {
			[sound stop];
			[indicator stopAnimation: self];
		}
		DESTROY (sound);
	}
}

- (BOOL)canDisplayPath:(NSString *)path
{
	NSDictionary *attributes;
	NSString *defApp, *fileType, *extension;
	NSArray *types;
//	OGGEditor *anOgg;

	attributes = [[NSFileManager defaultManager] fileAttributesAtPath: path
												 traverseLink: YES];
	if ([attributes objectForKey: NSFileType] == NSFileTypeDirectory) {
		return NO;
	}		
			
	[ws getInfoForFile: path application: &defApp type: &fileType];
	
	if(([fileType isEqual: NSPlainFileType] == NO)
	   && ([fileType isEqual: NSShellCommandFileType] == NO)) {
		return NO;
	}

	extension = [path pathExtension];
	types = [NSArray arrayWithObjects: @"ogg", @"mp3", @"flac", @"midi", @"voc", @"shn", @"mod", nil];

	if ([types containsObject: [extension lowercaseString]]) {

// Get Tag info
			
		NSString *str, *info;
		NSAttributedString *attributedString;
		NSFont *font;
			   
		NSTask *task = [[NSTask alloc] init];
		NSPipe *pipe = [[NSPipe alloc] init];
		NSFileHandle *fh = [pipe fileHandleForReading];

		tags = [[NSMutableArray alloc] init];

		if ([[extension lowercaseString] isEqual: @"ogg"]) {	

			anOgg = [[OGGEditor alloc] init];			
			[anOgg initWithFile: path];			
					
  int raw_seconds = [[anOgg seconds] intValue];
  int minutes = raw_seconds / 60;
  int seconds = raw_seconds - (minutes * 60);
  if(seconds < 10)
    time = [NSString stringWithFormat: @"%i:0%i",minutes,seconds];
  else
    time = [NSString stringWithFormat: @"%i:%i",minutes,seconds];
   
NSDictionary *artistDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Artist", @"tag",
        [anOgg artist], @"value", 
        nil];

NSDictionary *titleDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Title", @"tag",
        [anOgg title], @"value", 
        nil];

NSDictionary *albumDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Album", @"tag",
        [anOgg album], @"value", 
        nil];

NSDictionary *yearDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Year", @"tag",
        [anOgg year], @"value", 
        nil];

NSDictionary *trackDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Track No.", @"tag",
        [anOgg trackNumber], @"value", 
        nil];

NSDictionary *genreDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Genre", @"tag",
        [anOgg genre], @"value", 
        nil];

NSDictionary *timeDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Play Time", @"tag",
        time, @"value", 
        nil];

NSDictionary *commentDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Comments", @"tag",
        [anOgg comment], @"value", 
        nil];

NSDictionary *rateDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Rate", @"tag",
        [anOgg rate], @"value", 
        nil];

NSDictionary *channelsDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Channels", @"tag",
        [anOgg channels], @"value", 
        nil];

NSDictionary *nominalDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Nominal", @"tag",
        [anOgg nominalOfBitrate], @"value", 
        nil];

NSDictionary *upperDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Upper", @"tag",
        [anOgg upperOfBitrate], @"value", 
        nil];

NSDictionary *lowerDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Lower", @"tag",
        [anOgg lowerOfBitrate], @"value", 
        nil];

NSDictionary *vendorDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Vendor", @"tag",
        [anOgg vendor], @"value", 
        nil];


  [tags addObject: artistDict];
  [tags addObject: titleDict];
  [tags addObject: albumDict];
  [tags addObject: trackDict];
  [tags addObject: yearDict];
  [tags addObject: genreDict];
  [tags addObject: timeDict];
  [tags addObject: commentDict];
  [tags addObject: rateDict];
  [tags addObject: channelsDict];
  [tags addObject: nominalDict];
  [tags addObject: upperDict];
  [tags addObject: lowerDict];
  [tags addObject: vendorDict];
 
		} else if ([[extension lowercaseString] isEqual: @"mp3"]) {
			
			anMp3 = [[TagAPI alloc] init];
			[anMp3 examineFile: path];

NSDictionary *artistDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Artist", @"tag",
        [anMp3 getArtist], @"value", 
        nil];

NSDictionary *titleDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Title", @"tag",
        [anMp3 getTitle], @"value", 
        nil];

NSDictionary *albumDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Album", @"tag",
        [anMp3 getAlbum], @"value", 
        nil];

NSDictionary *yearDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Year", @"tag",
        [anMp3 getYear], @"value", 
        nil];

NSDictionary *trackDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Track No.", @"tag",
        [anMp3 getTrack], @"value", 
        nil];

NSDictionary *genreDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Genre", @"tag",
        [anMp3 getGenreNames], @"value", 
        nil];

NSDictionary *timeDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Play Time", @"tag",
        [anMp3 getDurationString], @"value", 
        nil];

NSDictionary *commentDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Comments", @"tag",
        [anMp3 getComments], @"value", 
        nil];

NSDictionary *rateDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Rate", @"tag",
        [anMp3 getFrequencyString], @"value", 
        nil];

NSDictionary *channelsDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Channels", @"tag",
        [anMp3 getChannelString], @"value", 
        nil];

NSDictionary *bitrateDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Bitrate", @"tag",
        [anMp3 getBitRateString], @"value", 
        nil];


NSDictionary *encoderDict = [[NSDictionary alloc] initWithObjectsAndKeys:
        @"Encoder", @"tag",
        [anMp3 getEncodedBy], @"value", 
        nil];

DESTROY (anMp3);

  [tags addObject: artistDict];
  [tags addObject: titleDict];
  [tags addObject: albumDict];
  [tags addObject: trackDict];
  [tags addObject: yearDict];
  [tags addObject: genreDict];
  [tags addObject: timeDict];
  [tags addObject: commentDict];
  [tags addObject: rateDict];
  [tags addObject: channelsDict];
  [tags addObject: bitrateDict];
  [tags addObject: encoderDict];

		} else {
			[task setLaunchPath: @"/usr/bin/file"];

		str = [NSString stringWithFormat: @"%@", path];

		[task setArguments: [NSArray arrayWithObjects: @"-b", str, nil]];

		[task setStandardOutput: pipe];
		[task launch];

		info=[[NSString alloc] initWithData:[fh readDataToEndOfFile]
							       encoding:NSASCIIStringEncoding]; 

		NSDictionary *infoDict = [[NSDictionary alloc] initWithObjectsAndKeys:
													   @"File Type", @"tag",
													   info, @"value", 
													   nil];
		[tags addObject: infoDict];
		} 

NSTableColumn *column;

if ([infoView numberOfColumns] < 2) {
	
column = [[NSTableColumn alloc] initWithIdentifier: @"tag"];
   [column setMinWidth: 80];
   [column setEditable: NO];
   [column setResizable: NO];
   [infoView addTableColumn: column];
   RELEASE(column);

column = [[NSTableColumn alloc] initWithIdentifier: @"value"];
   [column setMinWidth: 700];
   [column setEditable: NO];
   [column setResizable: YES];
   [infoView addTableColumn: column];
   RELEASE(column);
}
  
 [infoView setDataSource: tags];
 [infoView setAutoresizesAllColumnsToFit: YES];
		   						   
 return YES;
	}
		   
	return NO;
}

- (BOOL)canDisplayDataOfType:(NSString *)type
{
	return NO;
}

- (NSString *)winname
{
	return NSLocalizedString(@"SGSound Inspector", @"");	
}

- (NSString *)description
{
	return NSLocalizedString(@"This Inspector allow you to play an Ogg Vorbis or mp3 file", @"");	
}

@end
